@@ -7,20 +7,23 @@ https://github.com/google/neuroglancer/tree/master/src/neuroglancer/datasource/p
 
 This datasource contains the code for manipulating images.
 """
+from functools import reduce
 import itertools
+import operator
 import uuid
 
 import numpy as np
 from tqdm import tqdm
 
+from cloudfiles import CloudFiles
+
 from cloudvolume import lib, exceptions
-from ....lib import Bbox, Vec
-from .... import sharedmemory
-from ....storage import Storage
+from ....lib import Bbox, Vec, sip, first
+from .... import sharedmemory, chunks
 
 from ... import autocropfn, readonlyguard, ImageSourceInterface
 from .. import sharding
-from .common import chunknames
+from .common import chunknames, gridpoints, compressed_morton_code
 from . import tx, rx
 
 class PrecomputedImageSource(ImageSourceInterface):
@@ -55,6 +58,10 @@ class PrecomputedImageSource(ImageSourceInterface):
     """Unlink the current shared memory location from the filesystem."""
     return sharedmemory.unlink(self.shared_memory_id)
 
+  def grid_size(self, mip=None):
+    mip = mip if mip is not None else self.config.mip
+    return np.ceil(self.meta.volume_size(mip) / self.meta.chunk_size(mip)).astype(np.int64)
+
   def check_bounded(self, bbox, mip):
     if self.bounded and not self.meta.bounds(mip).contains_bbox(bbox):
       raise exceptions.OutOfBoundsError("""
@@ -74,6 +81,24 @@ class PrecomputedImageSource(ImageSourceInterface):
         )
       )
 
+  def has_data(self, mip=None):
+    """
+    Returns whether the specified mip appears to have data 
+    by testing whether the "folder" exists.
+
+    Returns: bool
+  
+    The mip is the index into the returned list. If
+    the entry is True, then the data appears to be there.
+    If the entry is False, then the data is not there.
+    """
+    mip = mip if mip is not None else self.config.mip
+    mip = self.meta.to_mip(mip)
+    
+    cf = CloudFiles(self.meta.cloudpath)
+    key = self.meta.key(mip)
+    return len(list(sip(cf.list(prefix=key), 1))) > 0
+
   def download(
       self, bbox, mip, parallel=1, 
       location=None, retain=False,
@@ -166,6 +191,16 @@ class PrecomputedImageSource(ImageSourceInterface):
     if location is None:
       location = self.shared_memory_id
 
+    if self.is_sharded(mip):
+      (filename, shard) = self.make_shard(image, bbox, mip)
+      basepath = self.meta.join(self.meta.cloudpath, self.meta.key(mip))
+      CloudFiles(basepath, progress=self.config.progress).put(
+        filename, shard, 
+        compress=self.config.compress, 
+        cache_control=self.config.cdn_cache
+      )
+      return
+
     return tx.upload(
       self.meta, self.cache,
       image, offset, mip,
@@ -196,15 +231,13 @@ class PrecomputedImageSource(ImageSourceInterface):
     )
     realized_bbox = Bbox.clamp(realized_bbox, self.meta.bounds(mip))
 
-    cloudpaths = list(chunknames(
+    cloudpaths = chunknames(
       realized_bbox, self.meta.bounds(mip), 
       self.meta.key(mip), self.meta.chunk_size(mip),
       protocol=self.meta.path.protocol
-    ))
+    )
 
-    with Storage(self.meta.cloudpath, progress=self.config.progress) as storage:
-      existence_report = storage.files_exist(cloudpaths)
-    return existence_report
+    return CloudFiles(self.meta.cloudpath, progress=self.config.progress).exists(cloudpaths)
 
   @readonlyguard
   def delete(self, bbox, mip=None):
@@ -230,19 +263,18 @@ class PrecomputedImageSource(ImageSourceInterface):
         bbox, realized_bbox
       ))
 
-    cloudpaths = list(chunknames(
+    cloudpaths = lambda: chunknames(
       realized_bbox, self.meta.bounds(mip),
       self.meta.key(mip), self.meta.chunk_size(mip),
       protocol=self.meta.path.protocol
-    ))
+    ) # need to regenerate so that generator isn't used up
 
-    with Storage(self.meta.cloudpath, progress=self.config.progress) as storage:
-      storage.delete_files(cloudpaths)
+    CloudFiles(self.meta.cloudpath, progress=self.config.progress) \
+      .delete(cloudpaths())
 
     if self.cache.enabled:
-      with Storage('file://' + self.cache.path, progress=self.config.progress) as storage:
-        storage.delete_files(cloudpaths)
-
+      CloudFiles('file://' + self.cache.path, progress=self.config.progress) \
+        .delete(cloudpaths())
 
   def transfer_to(self, cloudpath, bbox, mip, block_size=None, compress=True, compress_level=None):
     """
@@ -321,17 +353,88 @@ class PrecomputedImageSource(ImageSourceInterface):
       total=num_blocks,
     )
 
+    cfsrc = CloudFiles(self.meta.cloudpath)
+    cfdest = CloudFiles(cloudpath)
+
     with pbar:
-      with Storage(self.meta.cloudpath) as src_stor:
-        with Storage(cloudpath) as dest_stor:
-          for _ in range(num_blocks, 0, -1):
-            srcpaths = list(itertools.islice(cloudpaths, step))
-            files = src_stor.get_files(srcpaths)
-            files = [ (f['filename'], f['content']) for f in files ]
-            dest_stor.put_files(
-              files=files, 
-              compress=compress, 
-              compress_level=compress_level,
-              content_type=tx.content_type(destvol),
-            )
-            pbar.update()
+      for _ in range(num_blocks, 0, -1):
+        srcpaths = list(itertools.islice(cloudpaths, step))
+        files = cfsrc.get(srcpaths)
+        files = [ (f['path'], f['content']) for f in files ]
+        cfdest.puts(
+          files, 
+          compress=compress, 
+          compression_level=compress_level,
+          content_type=tx.content_type(destvol),
+        )
+        pbar.update()
+
+  def make_shard(self, img, bbox, mip=None, spec=None):
+    """
+    Convert an image that represents a single complete shard 
+    into a shard file.
+  
+    img: a volumetric numpy array image
+    bbox: the bbox it represents in voxel coordinates
+    mip: if specified, use the sharding specification from 
+      this mip level, otherwise use the sharding spec from
+      the current implicit mip level in config.
+    spec: use the provided specification (overrides mip parameter)
+
+    Returns: (filename, shard_file)
+    """
+    mip = mip if mip is not None else self.config.mip
+    scale = self.meta.scale(mip)
+
+    if spec is None:
+      if 'sharding' in scale:
+        spec = sharding.ShardingSpecification.from_dict(scale['sharding'])
+      else:
+        raise ValueError("mip {} does not have a sharding specification.".format(mip))
+
+    if bbox.subvoxel():
+      raise ValueError("Bounding box is too small to make a shard. Got: {}".format(bbox))
+
+    bbox = Bbox.create(bbox, self.meta.bounds(mip), bounded=True)
+    aligned_bbox = bbox.expand_to_chunk_size(
+      self.meta.chunk_size(mip), offset=self.meta.voxel_offset(mip)
+    )
+    aligned_bbox = Bbox.clamp(aligned_bbox, self.meta.bounds(mip))
+
+    if bbox != aligned_bbox:
+      raise exceptions.AlignmentError(
+        "Unable to create shard from a non-chunk aligned bounding box. Requested: {}, Aligned: {}".format(
+        bbox, aligned_bbox
+      ))
+
+    grid_size = self.grid_size(mip)
+    chunk_size = self.meta.chunk_size(mip)
+    reader = sharding.ShardReader(self.meta, self.cache, spec)
+
+    gpts = lambda: gridpoints(bbox, self.meta.bounds(mip), chunk_size)
+    all_same_shard = bool(reduce(lambda a,b: operator.eq(a,b) and a,
+      map(reader.get_filename,
+        map(lambda gpt: compressed_morton_code(gpt, grid_size), gpts())
+      )
+    ))
+
+    if not all_same_shard:
+      raise exceptions.AlignmentError(
+        "The gridpoints for this image did not all correspond to the same shard. Got: {}".format(bbox)
+      )
+
+    labels = {}
+    for pt in gpts():
+      cutout_bbx = Bbox( pt * chunk_size, (pt+1) * chunk_size )
+      chunk = img[ cutout_bbx.to_slices() ]
+      morton_code = compressed_morton_code(pt, grid_size)
+      labels[morton_code] = chunks.encode(chunk, self.meta.encoding(mip))
+
+    shard_filename = reader.get_filename(first(labels.keys()))
+
+    return (shard_filename, spec.synthesize_shard(labels))
+
+  def is_sharded(self, mip):
+    scale = self.meta.scale(mip)
+    return 'sharding' in scale and scale['sharding'] is not None
+
