@@ -10,9 +10,10 @@ from six import string_types
 from six.moves import range
 from tqdm import tqdm
 
+from cloudfiles import CloudFiles 
+
 from ... import exceptions
 from ...provenance import DatasetProvenance, DataLayerProvenance
-from ...storage import SimpleStorage
 
 from ... import lib
 from ...lib import ( 
@@ -191,8 +192,7 @@ class PrecomputedMetadata(object):
 
     Returns: dict
     """
-    with SimpleStorage(self.cloudpath) as stor:
-      info = stor.get_json('info')
+    info = CloudFiles(self.cloudpath).get_json('info')
 
     if info is None:
       raise exceptions.InfoUnavailableError(
@@ -299,12 +299,13 @@ Hops:
       indent=2, 
       separators=(',', ': ')
     )
-
-    with SimpleStorage(self.cloudpath) as stor:
-      stor.put_file('info', infojson, 
-        content_type='application/json', 
-        cache_control='no-cache'
-      )
+    # use put instead of put_json to preserve formatting
+    cf = CloudFiles(self.cloudpath)
+    cf.put(
+      'info', infojson,
+      cache_control='no-cache',
+      content_type='application/json'
+    )
 
     if self.cache:
       self.cache.maybe_cache_info()
@@ -359,30 +360,30 @@ Hops:
 
     Returns: dict
     """
-    with SimpleStorage(self.cloudpath) as stor:
-      provfile = stor.get_file('provenance')
-      if provfile:
-        provfile = provfile.decode('utf-8')
+    cf = CloudFiles(self.cloudpath)
+    provfile = cf.get('provenance')
+    if provfile:
+      provfile = provfile.decode('utf-8')
 
-        # The json5 decoder is *very* slow
-        # so use the stricter but much faster json 
-        # decoder first, and try it only if it fails.
+      # The json5 decoder is *very* slow
+      # so use the stricter but much faster json 
+      # decoder first, and try it only if it fails.
+      try:
+        provfile = json.loads(provfile)
+      except json.decoder.JSONDecodeError:
         try:
-          provfile = json.loads(provfile)
-        except json.decoder.JSONDecodeError:
-          try:
-            provfile = json5.loads(provfile)
-          except ValueError:
-            raise ValueError(red("""The provenance file could not be JSON decoded. 
-              Please reformat the provenance file before continuing. 
-              Contents: {}""".format(provfile)))
-      else:
-        provfile = {
-          "sources": [],
-          "owners": [],
-          "processing": [],
-          "description": "",
-        }
+          provfile = json5.loads(provfile)
+        except ValueError:
+          raise ValueError(red("""The provenance file could not be JSON decoded. 
+            Please reformat the provenance file before continuing. 
+            Contents: {}""".format(provfile)))
+    else:
+      provfile = {
+        "sources": [],
+        "owners": [],
+        "processing": [],
+        "description": "",
+      }
 
     return self._cast_provenance(provfile)
 
@@ -401,11 +402,14 @@ Hops:
       separators=(',', ': ')
     )
 
-    with SimpleStorage(self.cloudpath) as stor:
-      stor.put_file('provenance', prov, 
-        content_type='application/json',
-        cache_control='no-cache',
-      )
+    # need to use put vs put_json to preserve formatting
+    cf = CloudFiles(self.cloudpath)
+    cf.put(
+      'provenance', prov, 
+      cache_control='no-cache', 
+      content_type='application/json'
+    )
+
     if self.cache:
       self.cache.maybe_cache_provenance()
 
@@ -496,9 +500,28 @@ Hops:
     """e.g. 'uint8'"""
     return self.info['data_type']
 
-  def encoding(self, mip):
-    """e.g. 'raw' or 'jpeg'"""
-    return self.info['scales'][mip]['encoding']
+  def encoding(self, mip, encoding=None):
+    """
+    If encoding is provided, set the encoding for this
+    mip level. If the encoding is not provided, this is
+    a getter.
+
+    Typical values: 'raw', 'jpeg', 'compressed_segmentation'
+
+    Returns encoding for the mip level either way.
+    """
+    if encoding is None:
+      return self.info['scales'][mip]['encoding']
+
+    encoding = encoding.lower()
+    scale = self.scale(mip)
+    scale['encoding'] = encoding
+    if (encoding == 'compressed_segmentation' \
+      and 'compressed_segmentation_block_size' not in scale):
+
+      scale['compressed_segmentation_block_size'] = (8,8,8)
+
+    return encoding
 
   def compressed_segmentation_block_size(self, mip):
     if 'compressed_segmentation_block_size' in self.info['scales'][mip]:
@@ -556,6 +579,10 @@ Hops:
     """The subdirectory within the data layer containing the chunks for this mip level"""
     return self.scale(mip)['key']
 
+  @property
+  def keys(self):
+    return [ self.key(mip) for mip in self.available_mips ]
+
   def bounds(self, mip):
     """Returns a 3D spatial bounding box for the dataset with dimensions in voxels."""
     offset = self.voxel_offset(mip)
@@ -599,6 +626,10 @@ Hops:
     """Used for manually resetting downsamples if something messed up."""
     self.info['scales'] = self.info['scales'][0:1]
 
+  def add_resolution(self, res, encodig=None, chunk_size=None, info=None):
+    factor = Vec(res, dtype=np.float32) // self.meta.resolution(0)
+    return self.add_scale(factor, encoding, chunk_size, info)
+
   def add_scale(self, factor, encoding=None, chunk_size=None, info=None):
     """
     Generate a new downsample scale to for the info file and return an updated dictionary.
@@ -627,7 +658,7 @@ Hops:
     #    the mip 1 will have an offset of 5
     #    the mip 2 will have an offset of 2 instead of 2.5 
     #        meaning that it will be half a pixel to the left
-    if not chunk_size:
+    if chunk_size is None:
       chunk_size = lib.find_closest_divisor(fullres['chunk_sizes'][0], closest_to=[64,64,64])
 
     if encoding is None:
@@ -637,7 +668,7 @@ Hops:
       u"encoding": encoding,
       u"chunk_sizes": [ list(map(int, chunk_size)) ],
       u"resolution": list(map(int, Vec(*fullres['resolution']) * factor )),
-      u"voxel_offset": downscale(fullres['voxel_offset'], factor, np.floor),
+      u"voxel_offset": downscale(fullres.get('voxel_offset', (0,0,0)), factor, np.floor),
       u"size": downscale(fullres['size'], factor, np.ceil),
     }
 
