@@ -8,6 +8,7 @@ import sys
 import math
 import operator
 import time
+import types
 import random
 import string 
 from itertools import product
@@ -38,16 +39,44 @@ COLORS = {
 MACHINE_EPSILON = (7. / 3) - (4. / 3) - 1
 
 class NumpyEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, np.ndarray):
-            return obj.tolist()
-        if isinstance(obj, np.integer):
-            return int(obj)
-        if isinstance(obj, np.floating):
-            return float(obj)
-        return json.JSONEncoder.default(self, obj)
+  def default(self, obj):
+    if isinstance(obj, np.ndarray):
+      return obj.tolist()
+    if isinstance(obj, np.integer):
+      return int(obj)
+    if isinstance(obj, np.floating):
+      return float(obj)
+    return json.JSONEncoder.default(self, obj)
+
+def first(lst):
+  if isinstance(lst, types.GeneratorType):
+    return next(lst)
+  try:
+    return lst[0]
+  except TypeError:
+    return next(iter(lst))
+
+def sip(iterable, block_size):
+  """Sips a fixed size from the iterable."""
+  ct = 0
+  block = []
+  for x in iterable:
+    ct += 1
+    block.append(x)
+    if ct == block_size:
+      yield block
+      ct = 0
+      block = []
+
+  if len(block) > 0:
+    yield block
 
 def toiter(obj, is_iter=False):
+  if isinstance(obj, str) or isinstance(obj, dict):
+    if is_iter:
+      return [ obj ], False
+    return [ obj ]
+
   try:
     iter(obj)
     if is_iter:
