@@ -79,6 +79,11 @@ def schedule_jobs(
 
   Return: list of results
   """
+  if concurrency < 0:
+    raise ValueError("concurrency value cannot be negative: {}".format(concurrency))
+  elif concurrency == 0:
+    return [ fn() for fn in tqdm(fns, disable=(not progress), desc=progress) ]
+
   if green:
     return schedule_green_jobs(fns, concurrency, progress, total)
 
