@@ -40,15 +40,15 @@ def test_cloud_access():
 
 def test_fill_missing():
   info = CloudVolume.create_new_info(
-  num_channels=1, # Increase this number when we add more tests for RGB
-  layer_type='image', 
-  data_type='uint8', 
-  encoding='raw',
-  resolution=[ 1,1,1 ], 
-  voxel_offset=[0,0,0], 
-  volume_size=[128,128,64],
-  mesh='mesh', 
-  chunk_size=[ 64,64,64 ],
+    num_channels=1, # Increase this number when we add more tests for RGB
+    layer_type='image', 
+    data_type='uint8', 
+    encoding='raw',
+    resolution=[ 1,1,1 ], 
+    voxel_offset=[0,0,0], 
+    volume_size=[128,128,64],
+    mesh='mesh', 
+    chunk_size=[ 64,64,64 ],
   )
 
   vol = CloudVolume('file:///tmp/cloudvolume/empty_volume', mip=0, info=info)
@@ -64,36 +64,51 @@ def test_fill_missing():
   vol.cache.flush()
   delete_layer('/tmp/cloudvolume/empty_volume')
 
-def test_aligned_read():
-  for green in (False, True):
-    print("green", green)
-    delete_layer()
-    cv, data = create_layer(size=(50,50,50,1), offset=(0,0,0))
-    cv.green_threads = green
-    # the last dimension is the number of channels
-    assert cv[0:50,0:50,0:50].shape == (50,50,50,1)
-    assert np.all(cv[0:50,0:50,0:50] == data)
-    
-    delete_layer()
-    cv, data = create_layer(size=(128,64,64,1), offset=(0,0,0))
-    cv.green_threads = green
-    # the last dimension is the number of channels
-    assert cv[0:64,0:64,0:64].shape == (64,64,64,1) 
-    assert np.all(cv[0:64,0:64,0:64] ==  data[:64,:64,:64,:])
+def test_has_data():
+  delete_layer()
+  cv, data = create_layer(size=(50,50,50,1), offset=(0,0,0))
+  cv.add_scale((2,2,1))
 
-    delete_layer()
-    cv, data = create_layer(size=(128,64,64,1), offset=(10,20,0))
-    cv.green_threads = green
-    cutout = cv[10:74,20:84,0:64]
-    # the last dimension is the number of channels
-    assert cutout.shape == (64,64,64,1) 
-    assert np.all(cutout == data[:64,:64,:64,:])
-    # get the second chunk
-    cutout2 = cv[74:138,20:84,0:64]
-    assert cutout2.shape == (64,64,64,1) 
-    assert np.all(cutout2 == data[64:128,:64,:64,:])
+  assert cv.image.has_data(0) == True
+  assert cv.image.has_data(1) == False
+  assert cv.image.has_data([1,1,1]) == True
+  assert cv.image.has_data([2,2,1]) == False
 
-    assert cv[25, 25, 25].shape == (1,1,1,1)
+  try:
+    cv.image.has_data(2)
+    assert False
+  except exceptions.ScaleUnavailableError:
+    pass
+
+@pytest.mark.parametrize('green', (True, False))
+def test_aligned_read(green):
+  delete_layer()
+  cv, data = create_layer(size=(50,50,50,1), offset=(0,0,0))
+  cv.green_threads = green
+  # the last dimension is the number of channels
+  assert cv[0:50,0:50,0:50].shape == (50,50,50,1)
+  assert np.all(cv[0:50,0:50,0:50] == data)
+  
+  delete_layer()
+  cv, data = create_layer(size=(128,64,64,1), offset=(0,0,0))
+  cv.green_threads = green
+  # the last dimension is the number of channels
+  assert cv[0:64,0:64,0:64].shape == (64,64,64,1) 
+  assert np.all(cv[0:64,0:64,0:64] ==  data[:64,:64,:64,:])
+
+  delete_layer()
+  cv, data = create_layer(size=(128,64,64,1), offset=(10,20,0))
+  cv.green_threads = green
+  cutout = cv[10:74,20:84,0:64]
+  # the last dimension is the number of channels
+  assert cutout.shape == (64,64,64,1) 
+  assert np.all(cutout == data[:64,:64,:64,:])
+  # get the second chunk
+  cutout2 = cv[74:138,20:84,0:64]
+  assert cutout2.shape == (64,64,64,1) 
+  assert np.all(cutout2 == data[64:128,:64,:64,:])
+
+  assert cv[25, 25, 25].shape == (1,1,1,1)
 
 def test_save_images():
   delete_layer()
@@ -311,62 +326,60 @@ def test_autocropped_read():
   assert img.shape == (0,0,0,1)
   assert np.all(img == data[0:0, 0:0, 0:0])    
 
-def test_download_upload_file():
-  for green in (False, True):
-    print("green:", green)
-    delete_layer()
-    cv, _ = create_layer(size=(50,50,50,1), offset=(0,0,0))
-    cv.green_threads = green
+@pytest.mark.parametrize('green', (True, False))
+def test_download_upload_file(green):
+  delete_layer()
+  cv, _ = create_layer(size=(50,50,50,1), offset=(0,0,0))
+  cv.green_threads = green
 
-    mkdir('/tmp/file/')
+  mkdir('/tmp/file/')
 
-    cv.download_to_file('/tmp/file/test', cv.bounds)
-    cv2 = CloudVolume('file:///tmp/file/test2/', info=cv.info)
-    cv2.upload_from_file('/tmp/file/test', cv.bounds)
+  cv.download_to_file('/tmp/file/test', cv.bounds)
+  cv2 = CloudVolume('file:///tmp/file/test2/', info=cv.info)
+  cv2.upload_from_file('/tmp/file/test', cv.bounds)
 
-    assert np.all(cv2[:] == cv[:])
-    shutil.rmtree('/tmp/file/')
+  assert np.all(cv2[:] == cv[:])
+  shutil.rmtree('/tmp/file/')
 
-def test_write():
-  for green in (False, True):
-    print("green:", green)
-    delete_layer()
-    cv, _ = create_layer(size=(50,50,50,1), offset=(0,0,0))
-    cv.green_threads = green
+@pytest.mark.parametrize('green', (True, False))
+def test_write(green):
+  delete_layer()
+  cv, _ = create_layer(size=(50,50,50,1), offset=(0,0,0))
+  cv.green_threads = green
 
-    replacement_data = np.zeros(shape=(50,50,50,1), dtype=np.uint8)
-    cv[0:50,0:50,0:50] = replacement_data
-    assert np.all(cv[0:50,0:50,0:50] == replacement_data)
+  replacement_data = np.zeros(shape=(50,50,50,1), dtype=np.uint8)
+  cv[0:50,0:50,0:50] = replacement_data
+  assert np.all(cv[0:50,0:50,0:50] == replacement_data)
 
-    replacement_data = np.random.randint(255, size=(50,50,50,1), dtype=np.uint8)
-    cv[0:50,0:50,0:50] = replacement_data
-    assert np.all(cv[0:50,0:50,0:50] == replacement_data)
+  replacement_data = np.random.randint(255, size=(50,50,50,1), dtype=np.uint8)
+  cv[0:50,0:50,0:50] = replacement_data
+  assert np.all(cv[0:50,0:50,0:50] == replacement_data)
 
-    replacement_data = np.random.randint(255, size=(50,50,50,1), dtype=np.uint8)
-    bbx = Bbox((0,0,0), (50,50,50))
-    cv[bbx] = replacement_data
-    assert np.all(cv[bbx] == replacement_data)
+  replacement_data = np.random.randint(255, size=(50,50,50,1), dtype=np.uint8)
+  bbx = Bbox((0,0,0), (50,50,50))
+  cv[bbx] = replacement_data
+  assert np.all(cv[bbx] == replacement_data)
 
-    # out of bounds
-    delete_layer()
-    cv, _ = create_layer(size=(128,64,64,1), offset=(10,20,0))
-    cv.green_threads = green
-    with pytest.raises(ValueError):
-      cv[74:150,20:84,0:64] = np.ones(shape=(64,64,64,1), dtype=np.uint8)
-    
-    # non-aligned writes
-    delete_layer()
-    cv, _ = create_layer(size=(128,64,64,1), offset=(10,20,0))
-    cv.green_threads = green
-    with pytest.raises(ValueError):
-      cv[21:85,0:64,0:64] = np.ones(shape=(64,64,64,1), dtype=np.uint8)
+  # out of bounds
+  delete_layer()
+  cv, _ = create_layer(size=(128,64,64,1), offset=(10,20,0))
+  cv.green_threads = green
+  with pytest.raises(ValueError):
+    cv[74:150,20:84,0:64] = np.ones(shape=(64,64,64,1), dtype=np.uint8)
+  
+  # non-aligned writes
+  delete_layer()
+  cv, _ = create_layer(size=(128,64,64,1), offset=(10,20,0))
+  cv.green_threads = green
+  with pytest.raises(ValueError):
+    cv[21:85,0:64,0:64] = np.ones(shape=(64,64,64,1), dtype=np.uint8)
 
-    # test bounds check for short boundary chunk
-    delete_layer()
-    cv, _ = create_layer(size=(25,25,25,1), offset=(1,3,5))
-    cv.green_threads = green
-    cv.info['scales'][0]['chunk_sizes'] = [[ 11,11,11 ]]
-    cv[:] = np.ones(shape=(25,25,25,1), dtype=np.uint8)
+  # test bounds check for short boundary chunk
+  delete_layer()
+  cv, _ = create_layer(size=(25,25,25,1), offset=(1,3,5))
+  cv.green_threads = green
+  cv.info['scales'][0]['chunk_sizes'] = [[ 11,11,11 ]]
+  cv[:] = np.ones(shape=(25,25,25,1), dtype=np.uint8)
 
 def test_non_aligned_write():
   delete_layer()
