@@ -710,12 +710,32 @@ def test_sharded():
 
       labels = []
       for fname in files.keys():
-        lbls = cv.skeleton.reader.list_labels(fname)
+        lbls = cv.skeleton.reader.list_labels(fname, path='skeletons')
         labels += list(lbls)
       
       labels.sort()
       assert labels == list(range(10))
 
+      for filename, shard in files.items():
+        decoded_skels = cv.skeleton.reader.disassemble_shard(shard)
+        for label, binary in decoded_skels.items():
+          Skeleton.from_precomputed(binary)
+
+      exists = cv.skeleton.reader.exists(list(range(11)), path='skeletons')
+      assert exists == {
+        0: 'skeletons/0.shard', 
+        1: 'skeletons/0.shard', 
+        2: 'skeletons/0.shard', 
+        3: 'skeletons/0.shard', 
+        4: 'skeletons/0.shard', 
+        5: 'skeletons/0.shard', 
+        6: 'skeletons/0.shard', 
+        7: 'skeletons/0.shard', 
+        8: 'skeletons/1.shard', 
+        9: 'skeletons/1.shard',
+        10: None,
+      }
+
   shutil.rmtree('/tmp/removeme/skeletons')
 
 
